import java.applet.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * The main applet for A8 415.101FC 2000
 * @author Vincent Wing Hei, Ho
 * @version 12.05.2000
 */

public class A8 extends Applet implements ActionListener, MouseListener {
    /** The green surface for the table. */
    public static final Color GREEN = new Color(0, 120, 40);
    /** The top boundary for the ball area. */ 
    public static final int TOP = 30;   
    /** The left boundary for the ball area. */
    public static final int LEFT = 5;  
    /** The bottom boundary for the ball area. */
    public static final int BOTTOM = 295;
    /** The right boundary for the ball area. */
    public static final int RIGHT = 395;   
    /** TextField for numberOfBalls */  
    private TextField numberOfBallsField;  
    /** Value in numberOfBalls textfield */  
    private int numberOfBalls = 0; 
    /** The friction textfield */
    private double friction = 1.0;   
    /** Array for all the balls */
    private Ball[] allBalls;
    /** Image object for flicker elimination */
    private Image offScreenImage;
    /** TextField for friction */  
    private TextField frictionField; 

    /**
     * initialise the applet
     */
    public void init() {
        addMouseListener(this);

        add (new Label("Number of Balls"));
        numberOfBallsField = new TextField(4);
        add (numberOfBallsField);
        numberOfBallsField.setText(String.valueOf(numberOfBalls));
        numberOfBallsField.addActionListener(this);

        add (new Label("Friction"));
        frictionField = new TextField(7);
        add (frictionField);
        frictionField.setText(String.valueOf(friction));
        frictionField.addActionListener(this);
        
        offScreenImage = createImage(400,300);

        createBalls();
        }
    /**
     * Helper method to create the balls
     */
    public void createBalls() {
        allBalls = new Ball[numberOfBalls + 1];
        for (int i = 0; i <= numberOfBalls; i++){
            allBalls[i] = new Ball(allBalls, i, friction);
        }
        System.out.println("Balls created: " + allBalls.length);
    }
    
    /** 
     * To detech user input from the textfields
     * @param e action event
     */     
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == numberOfBallsField) {
            numberOfBalls = (int)Double.valueOf(numberOfBallsField.getText()).doubleValue();
            if (numberOfBalls <= 0)    
                numberOfBalls = 0;    
            if (numberOfBalls >= 200)   
                numberOfBalls = 200;
            numberOfBallsField.setText(String.valueOf(numberOfBalls));
            createBalls();
        }

        if (e.getSource() == frictionField) {
            friction = Double.valueOf(frictionField.getText()).doubleValue();
            if (friction >= 1.01)
                friction = 1.01;
            if (friction <= 0.99)
                friction = 0.99;       
            frictionField.setText(String.valueOf(friction));

            for (int i = numberOfBalls; i >= 0; i--){
                allBalls[i].setFriction(friction);
            }
        }

    }

    /** 
     * To detect pressing of mouse button
     * @param e action event
     */
    public void  mousePressed(MouseEvent e) {   
        int mouseX = e.getX();     
        int mouseY = e.getY();	
        allBalls[0].setMousePos(mouseX, mouseY);
    }

    /**
     *  Paint method for drawing table and balls
     *  @param g the graphics object
     */
    public void paint(Graphics g) {    
        g.setColor(GREEN);
        g.fillRect(LEFT,TOP,RIGHT-LEFT,BOTTOM-TOP);   
        g.setColor(Color.black);
        g.drawRect(LEFT, TOP, RIGHT-LEFT, BOTTOM-TOP); 

        for (int i = numberOfBalls; i >= 0; i--){
            allBalls[i].move();
            allBalls[i].draw(g);
            
            for (int j = 0; j < numberOfBalls; j++) {
                allBalls[i].collide(allBalls[j]);
            }
        }
        repaint();
    }

    /** 
     *  Updater mathod, to help decrease flickering
     *  @param g the graphics object	
     */
    public void update(Graphics g) {
        Graphics offScreenGraphics = offScreenImage.getGraphics();
        offScreenGraphics.setColor(Color.white);
        offScreenGraphics.fillRect(0,0,400,300);
        paint(offScreenGraphics);
        g.drawImage(offScreenImage,0,0,this);
    }

    public void mouseClicked(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
}