import java.awt.*;

/** 
 * A ball class for A8 415.101FC 2000
 * @author Vincent Wing Hei, Ho
 * @version 12.05.2000
 */
public class Ball extends Object {
  	/** The radius of a ball. */
    private static final int RADIUS = 5;
    /** X-position of the centre of the ball */
    private double x;
    /** Y-position of the centre of the ball */
    private double y;
    /** X-velocity of the ball */
    private double vx;
    /** Y-velocity of the ball */
    private double vy;
    /** Stores the friction value */
    private double frictionValue;
    /** Array of balls for position check */
    private Ball[] allBalls;
    /** Index position inside allBalls array */
    private int index;
    
    /**
     * Object creater for the Balls
     */
    public Ball(Ball[] aBall, int i, double friction) {
        allBalls = aBall;
        index = i;
        frictionValue = friction;
     
        genPosition();
        while (isInvalid()) {
            genPosition();
        }
        System.out.println("Ball " + index + " created at (" + x +", " + y+") ");
    }
    
    public void genPosition() {
            x = Math.random() * 380 + RADIUS + 5;
            y = Math.random() * 255 + RADIUS + 30;
    }
    
    /**
     * Check whether a ball's position is overlapping with others
     * @return suitable boolean so new position is generated
     */
    public boolean isInvalid() {
        boolean isInvalid = false;
        for (int i = index - 1; i >= 0; i--) {
            if (allBalls[i].x > (x - RADIUS * 2.5)  &&  allBalls[i].x < (x + RADIUS * 2.5)
                    && (allBalls[i].y > (y - RADIUS * 2.5)  &&  allBalls[i].y < (y + RADIUS * 2.5))) {
                isInvalid = true;
                System.out.println("Ball " + index + " has to be regenerated");
            }
        }
        return isInvalid;
    }

    /**
     * Move the ball accordingly, taking care of 
     * the frictions and wall bouncing
     */
    public void move(){
        vx = vx * frictionValue;
        vy = vy * frictionValue;
        
  	    x = x + vx;
  	    y = y + vy;

  	    if (x + RADIUS >= 395 || x - RADIUS <= 6) {
  	        vx = -vx;
            if (x + RADIUS >=395)
                x = 394 - RADIUS;
            if (x - RADIUS <= 6)
                x = 7 + RADIUS;
	    }
	
	    if (y + RADIUS >= 295 || y - RADIUS <= 31) {
	        vy = -vy; 
            if (y + RADIUS >=295)
                y = 294 - RADIUS;
            if (y - RADIUS <= 31)
                y = 32 + RADIUS;
        }
    }
    
    /**
     * Set mouse position information to white ball
     * so it will move accordingly
     * @param mouseX X-position of the mouse
     * @param mouseY Y-position of the mouse
     */
    public void setMousePos(int mouseX, int mouseY) {
   	    vx = (mouseX - x) / 100;
   	    vy = (mouseY - y) / 100;
    }

    /**
     * Accept friction value and store into 
     * each ball's variable
     * @param friction the friction value
     */
    public void setFriction(double friction) {
        frictionValue = friction;
    }
    
    /**
     * Taking care of balls drawing
     * @param g the graphics object
     */
    public void draw(Graphics g) {
        if (index == 0)
            g.setColor(Color.white);
        else
            g.setColor(Color.red);
    
        g.fillOval((int)(x-RADIUS),(int)(y-RADIUS),2 * RADIUS,2 * RADIUS);
    }

    /**
    	Calculates the square of the distance between two points.
    	@param x0 the x coordinate of the first point
    	@param y0 the y coordinate of the first point
    	@param x1 the x coordinate of the second point
    	@param y1 the y coordinate of the second point
    	@return the square of the distance
    */
    private double distanceSquared(double x0, double y0, double x1, double y1) {
        return Math.pow(y1 - y0,2) + Math.pow(x1 - x0,2);
    }

    /**
    	Handles the collision of this ball with another.
    	Changes the velocities of both balls.
    	@param other the other ball
    */
    public void collide(Ball other) {
        double oX = other.x;
        double oY = other.y;
        double oVX = other.vx;
        double oVY = other.vy;
        if (distanceSquared(x, y, oX, oY) <= 4 * RADIUS * RADIUS) {
            double angleBetween = Math.atan2(oY - y, oX - x);
            double cos = Math.cos(-angleBetween);
            double sin = Math.sin(-angleBetween);
            double vxt = vx * cos - vy * sin;
            double vyt = vx * sin + vy * cos;
            double oVXt = oVX * cos - oVY * sin;
            double oVYt = oVX * sin + oVY * cos;
            cos = Math.cos(angleBetween);
            sin = Math.sin(angleBetween);
            vx = oVXt * cos - vyt * sin;
            vy = oVXt * sin + vyt * cos;
            other.vx = vxt * cos - oVYt * sin;
            other.vy = vxt * sin + oVYt * cos;
        }
    }
}
